#include <iostream>
#include "matrix.h"

using namespace std;

// Konstruktor: beállítja a mátrix méretét és allokálja a szükséges területet a HEAP-en.
Matrix::Matrix(int n, int m)
{
	_n = n;
	_m = m;
	
	_matrix = new int*[_n];
	for(int i = 0; i < _n; ++i)
		_matrix[i] = new int[_m];
}

// Destruktor, a HEAP-en lefoglalt terület felszabadítása.
Matrix::~Matrix()
{
	for(int i = 0; i < _n; ++i)
		delete[] _matrix[i];
	delete[] _matrix;
}

// A mátrix feltöltése egy adott értékkel.
void Matrix::fill(int v)
{
	for(int i = 0; i < _n; ++i)
		for(int j = 0; j < _m; ++j)
			_matrix[i][j] = v;
}

// Egy adott pozíció értékét meghatározza. Ha a pozíció a mátrixon kívül esik, akkor kivétel kiváltása.
void Matrix::insert(int i, int j, int v) throw(Exception)
{
	if(i >= 0 && i < _n && j >= 0 && j < _m)
		_matrix[i][j] = v;
	else
		throw OutOfBoundary;
}

// Egy adott pozíció értékének visszaadása. Ha a pozíció a mátrixon kívül esik, akkor kivétel kiváltása.
int Matrix::get(int i, int j) const throw(Exception)
{
	if(i >= 0 && i < _n && j >= 0 && j < _m)
		return _matrix[i][j];
	else
		throw OutOfBoundary;
}

// A mátrix kiírása a standard outputra.
void Matrix::print() const
{
	for(int i = 0; i < _n; ++i)
	{
		for(int j = 0; j < _m; ++j)
			cout << _matrix[i][j] << " ";
		cout << endl;
	}
}

// Összeadás operátor: összeadja a jelen és a paraméter mátrixot és egy harmadik objektumként visszaadja.
// Ha a két mátrix mérete nem egyezik, akkor kivétel kiváltása.
Matrix Matrix::operator+(const Matrix &om) const throw(Exception)
{
	if(_n == om._n && _m == om._m)
	{
		Matrix r(_n, _m);
		for(int i = 0; i < _n; ++i)
			for(int j = 0; j < _m; ++j)
				r.insert(i, j, _matrix[i][j] + om._matrix[i][j]);
		return r;
	}
	else
		throw SizeNotMatch;	
}
