#ifndef MATRIX_H
#define MATRIX_H

// Mátrix osztály.
class Matrix
{
	// Publikus rész (implementáció).
	public:
		// Kivételek.
		enum Exception
		{
			// Azt az esetet jelöli, ha a megadott index kívül esett a mátrixon.
			OutOfBoundary,
			// Azt az esetet jelöli, ha a paraméter mátrixok mérete nem egyezik, pedig ez a művelethez szükséges lenne.
			SizeNotMatch
		};
		
		// Konstruktor: beállítja a mátrix méretét és allokálja a szükséges területet a HEAP-en.
		Matrix(int n, int m);
		// Destruktor, a HEAP-en lefoglalt terület felszabadítása.
		virtual ~Matrix();
	
		// Mátrix méretének lekérdezése.
		int N() const { return _n; }
		int M() const { return _m; }
		
		// A mátrix feltöltése egy adott értékkel.
		void fill(int v);
		// Egy adott pozíció értékét meghatározza. Ha a pozíció a mátrixon kívül esik, akkor kivétel kiváltása.
		void insert(int i, int j, int v) throw(Exception);
		// Egy adott pozíció értékének visszaadása. Ha a pozíció a mátrixon kívül esik, akkor kivétel kiváltása.
		int get(int i, int j) const throw(Exception);
		// A mátrix kiírása a standard outputra.
		void print() const;
		
		// Összeadás operátor: összeadja a jelen és a paraméter mátrixot és egy harmadik objektumként visszaadja.
		// Ha a két mátrix mérete nem egyezik, akkor kivétel kiváltása.
		Matrix operator+(const Matrix &om) const throw(Exception);
	
	// Privát rész (reprezentáció).
	private:
		// A mátrix mérete.
		int _n, _m;
		// A mátrix elemei.
		int** _matrix;
};

#endif

