#include <iostream>
#include "menu.h"

using namespace std;

Menu::Menu()
{
	cout << "Udvozollek a Matrix tesztelo alkalmazasban!" << endl;
	
	for(int i = 0; i < sizeof(_matrices) / sizeof(Matrix*); ++i)
		_matrices[i] = NULL;
}

void Menu::run()
{
	int option = 0;
	do
	{
		menuWrite();
		option = getInteger("Valasztott menupont: ");

		switch(option)
		{
			case 1:
				testCreate();
				break;
			case 2:
				testFill();
				break;
			case 3:
				testInsert();
				break;
			case 4:
				testGet();
				break;
			case 5:
				testPrint();
				break;
			case 6:
				testAdd();
				break;
		}
	}
	while(option != 0);
}

void Menu::menuWrite() const
{
	cout << endl << endl;
	cout << "--- MENU ---" << endl;
	cout << "Valassz az alabbi opciok kozul!" << endl;
	cout << "\t1 - Matrix letrehozasa" << endl;
	cout << "\t2 - Matrix feltoltese egy ertekkel" << endl;
	cout << "\t3 - Matrix egy elemenek beallitasa" << endl;
	cout << "\t4 - Matrix egy elemenek lekerdezese" << endl;
	cout << "\t5 - Matrix kiiratasa" << endl;
	cout << "\t6 - Matrixok osszeadasa" << endl;
	cout << "\t0 - Kilepes" << endl;
}

int Menu::getInteger(const string& msg) const
{
	int number;
	bool error;
	do
	{
		cout << msg;
		cin >> number;
		if(cin.fail())
		{
			cin.clear();
			cin.ignore(256, '\n');
			cout << "A megadott ertek nem egesz szam!" << endl;
			error = true;
        }
		else
            error = false;
    }
    while(error);
	return number;
}

int Menu::getMatrixId() const
{
	int id = 0;
	do
	{
		id = getInteger("Matrix sorszama (1, 2): ");
	}
	while(id < 1 || id > 2);
	return id - 1;
}

void Menu::testCreate()
{
	cout << endl;
	cout << "-- Matrix letrehozasa --" << endl;
	
	Matrix* &matrix = _matrices[getMatrixId()];
	int n = getInteger("n := ");
	int m = getInteger("m := ");
	
	if(matrix != NULL)
		delete matrix;
	matrix = new Matrix(n, m);
	cout << "A matrix sikeresen letrehozva." << endl;
}

void Menu::testFill()
{
	cout << endl;
	cout << "-- Matrix feltoltese --" << endl;
	
	Matrix* &matrix = _matrices[getMatrixId()];
	if(matrix == NULL)
	{
		cout << "Elobb hozd letre a matrixot!" << endl;
		return;
	}
	int v = getInteger("v := ");	
	matrix->fill(v);
	cout << "A matrix sikeresen feltoltve az adott ertekkel." << endl;
}

void Menu::testInsert()
{
	cout << endl;
	cout << "-- Elem beallitasa --" << endl;
	
	Matrix* &matrix = _matrices[getMatrixId()];
	if(matrix == NULL)
	{
		cout << "Elobb hozd letre a matrixot!" << endl;
		return;
	}
	
	int i = getInteger("i := ");
	int j = getInteger("j := ");
	int v = getInteger("v := ");
	try
	{
		matrix->insert(i, j, v);
		cout << "Az elem erteke sikeresen beallitasra kerult." << endl;
	}
	catch(Matrix::Exception ex)
	{
		if(ex == Matrix::OutOfBoundary)
			cout << "Hiba: kivul indexeltel a matrixon!" << endl;
		else
			cout << "Nem vart hiba!" << endl;
	}
}

void Menu::testGet()
{
	cout << endl;
	cout << "-- Elem lekerdezese --" << endl;
	
	Matrix* &matrix = _matrices[getMatrixId()];
	if(matrix == NULL)
	{
		cout << "Elobb hozd letre a matrixot!" << endl;
		return;
	}
	
	int i = getInteger("i := ");
	int j = getInteger("j := ");
	try
	{
		int v = matrix->get(i, j);
		cout << "Az elem erteke: " << v << endl;
	}
	catch(Matrix::Exception ex)
	{
		if(ex == Matrix::OutOfBoundary)
			cout << "Hiba: kivul indexeltel a matrixon!" << endl;
		else
			cout << "Nem vart hiba!" << endl;
	}
}

void Menu::testPrint()
{
	cout << endl;
	cout << "-- Matrix kiiratasa --" << endl;
	
	Matrix* &matrix = _matrices[getMatrixId()];
	if(matrix == NULL)
	{
		cout << "Elobb hozd letre a matrixot!" << endl;
		return;
	}
	matrix->print();
}

void Menu::testAdd()
{
	cout << endl;
	cout << "-- Matrixok osszeadasa --" << endl;
	
	if(_matrices[0] == NULL || _matrices[1] == NULL)
	{
		cout << "Elobb hozd letre mindket matrixot!" << endl;
		return;
	}
	try
	{
		Matrix matrix = *_matrices[0] + *_matrices[1];
		matrix.print();
	}
	catch(Matrix::Exception ex)
	{
		if(ex == Matrix::SizeNotMatch)
			cout << "Hiba: az operandus matrixok merete nem egyezik!" << endl;
		else
			cout << "Nem vart hiba!" << endl;
	}
}
